/*
	Schrodinger's Box Haven  code version -0.1 alpha
        Copyright (C) 1994, Gordon Chan

	You can do whatever ya want with this code as long as you leave this
        copyright & disclaimer intact.  You can hack it up all you want, etc...
        No guarantees.

	Send comments, flames, bug reports, etc ... to
		au233@freenet.carleton.ca  <---- (preferred address for now)
		gjc@achilles.net
		gchan@ccs.carleton.ca
*/


/* Here's the copyrights & disclaimers from the Opium Den and unixCB codes */

/* Opium Den */
/*
  val's haven code v -0.85

        Copyright (C) 1994 Joel Ward

        Warranty:
                Don't blame me for anything.

        You can do whatever the fuck you want with this code as long as you
don't remove this copyright/disclaimer notice.
*/

/* unixCB */
/*
  
  cbd.c
  
  "cbd.c" is the main code for the CB simulator.
  
  Copyright (c) 1992, Gary Grossman.  All rights reserved.
  Send comments and questions to: garyg@soda.berkeley.edu
  
  */

void get_out();
void version();
void show_help ();
void toggle_gag ();
void toggle_reverse_gag ();
int num_of_annoy ();
void set_login ();
void login ();
void password ();


/* macros used in gag routines */

#define bitarrsize(n) \
	(((n)-1)/8+1)
#define clrbit(arr, n) \
	arr[(n)/8] &= ~(0x80 >> ((n)%8))
#define togbit(arr,n) \
	arr[(n)/8] ^= (0x80 >> ((n)%8))
#define testbit(arr,n) \
	(arr[(n)/8] & (0x80 >> ((n)%8)))


#define MAXPASSWD 9
